@@ -1,8 +1,8 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "dall-e-shell" "1.20.1"
+(define-package "dall-e-shell" "20250821.1033"
   "Interaction mode for DALL-E."
   '((emacs       "27.1")
-    (shell-maker "0.58.1"))
+    (shell-maker "0.79.1"))
   :url "https://github.com/xenodium/chatgpt-shell"
-  :commit "a24c003c79a720eafffc79bc3885070735f667f6"
-  :revdesc "a24c003c79a7")
+  :commit "428125f9fa8578703a9ca85d173b2cc9a3eb16b9"
+  :revdesc "428125f9fa85")
