@@ -4,9 +4,9 @@
 
 ;; Author: Alvaro Ramirez https://xenodium.com
 ;; URL: https://github.com/xenodium/chatgpt-shell
-;; Package-Version: 1.20.1
-;; Package-Revision: a24c003c79a7
-;; Package-Requires: ((emacs "27.1") (shell-maker "0.58.1"))
+;; Package-Version: 20250821.1033
+;; Package-Revision: 428125f9fa85
+;; Package-Requires: ((emacs "27.1") (shell-maker "0.79.1"))
 
 ;; This package is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -97,6 +97,13 @@ See `shell-maker-welcome-message' as an example."
   :type 'function
   :group 'dall-e-shell)
 
+(defcustom dall-e-shell-auth-header
+  (lambda ()
+    (format "Authorization: Bearer %s" (dall-e-shell-openai-key)))
+  "Function to generate the request's `Authorization' header string."
+  :type '(function :tag "Function")
+  :group 'chatgpt-shell)
+
 (defvaralias 'dall-e-shell-display-function 'shell-maker-display-function)
 
 (defvaralias 'dall-e-shell-read-string-function 'shell-maker-read-string-function)
@@ -126,7 +133,7 @@ or
       :url dall-e-shell--url
       :data (dall-e-shell--make-payload command)
       :headers (list "Content-Type: application/json; charset=utf-8"
-                     (funcall chatgpt-shell-auth-header))
+                     (funcall dall-e-shell-auth-header))
       :filter #'dall-e-shell--extract-response
       :shell shell))))
 
@@ -148,18 +155,20 @@ With NEW-SESSION, start a new session."
                        dall-e-shell-welcome-function
                        new-session
                        (when (dall-e-shell--shell-buffers)
-                         (buffer-name (seq-first (dall-e-shell--shell-buffers)))))
-    (with-current-buffer
-        ;; TODO: Add menus. See `chatgpt-shell--add-menus'.
-        (dall-e-shell--update-prompt t))
-    (define-key dall-e-shell-mode-map (kbd "C-c C-v")
-                #'dall-e-shell-swap-model-version)))
+                         (buffer-name (seq-first (dall-e-shell--shell-buffers)))))))
+
+(defvar-keymap dall-e-shell-mode-map
+  :parent shell-maker-mode-map
+  :doc "Keymap for `dall-e-shell-mode'."
+  "C-c C-v" #'dall-e-shell-swap-model-version)
+
+(shell-maker-define-major-mode dall-e-shell--config dall-e-shell-mode-map)
 
 (defun dall-e-shell--update-prompt (rename-buffer)
   "Update prompt and prompt regexp from `dall-e-shell-model-versions'.
 
 Set RENAME-BUFFER to also rename the buffer accordingly."
-  (unless (eq major-mode 'dall-e-shell-mode)
+  (unless (derived-mode-p 'dall-e-shell-mode)
     (user-error "Not in a shell"))
   (shell-maker-set-prompt
    (car (dall-e-shell--prompt-pair))
@@ -209,7 +218,7 @@ Set RENAME-BUFFER to also rename the buffer accordingly."
 (defun dall-e-shell-swap-model-version ()
   "Swap model version from `dall-e-shell-model-versions'."
   (interactive)
-  (unless (eq major-mode 'dall-e-shell-mode)
+  (unless (derived-mode-p 'dall-e-shell-mode)
     (user-error "Not in a shell"))
   (setq-local dall-e-shell-model-version
               (completing-read "Model version: "
@@ -231,9 +240,8 @@ Set RENAME-BUFFER to also rename the buffer accordingly."
       (if (equal (dall-e-shell-model-version) "dall-e-3")
           (push `(quality . ,dall-e-shell-image-quality) request-data)
         (user-error "`dall-e-shell-image-quality' must be used with \"dall-e-3\"")))
-    (when dall-e-shell-model-version
-      (push `(model . ,dall-e-shell-model-version)
-            request-data))
+    (push `(model . ,(dall-e-shell-model-version))
+          request-data)
     request-data))
 
 (defun dall-e-shell-interrupt (ignore-item)
@@ -243,7 +251,7 @@ With prefix IGNORE-ITEM, do not mark as failed."
   (interactive "P")
   (with-current-buffer
       (cond
-       ((eq major-mode 'dall-e-shell-mode)
+       ((derived-mode-p 'dall-e-shell-mode)
         (current-buffer))
        (t
         (shell-maker-buffer-name dall-e-shell--config)))
@@ -254,11 +262,11 @@ With prefix IGNORE-ITEM, do not mark as failed."
   (make-directory dall-e-shell-image-output-directory t)
   dall-e-shell-image-output-directory)
 
-(defun dall-e-shell--extract-response (raw-response &optional no-download)
-  "Extract DALL-E response from RAW-RESPONSE.
+(defun dall-e-shell--extract-response (object &optional no-download)
+  "Extract DALL-E response from OBJECT.
 Set NO-DOWNLOAD to skip automatic downloading."
   (if-let* ((buffer (shell-maker-buffer shell-maker--config))
-            (whole (shell-maker--json-parse-string raw-response))
+            (whole (shell-maker--json-parse-string (map-elt object :pending)))
             (url (let-alist whole
                    (let-alist (seq-first .data)
                      .url)))
@@ -280,8 +288,7 @@ Set NO-DOWNLOAD to skip automatic downloading."
             (propertize path 'display "[downloading...]")
           (concat (propertize path 'display "[downloading...]")
                   (format "\n\n%s" revised-prompt))))
-    (list (cons :filtered nil)
-          (cons :pending raw-response))))
+    object))
 
 (defun dall-e-shell-start-download (shell-buffer url path revised-prompt)
   "Start downloading image from URL into PATH and modify SHELL-BUFFER."
@@ -414,6 +421,18 @@ ERROR-CALLBACK otherwise."
                                     (funcall error-callback output))
                                   (kill-buffer output-buffer)))))))))
 
+(defun dall-e-shell-openai-key ()
+  "Get the OpenAI DALL-E key."
+  (cond ((stringp dall-e-shell-openai-key)
+         dall-e-shell-openai-key)
+        ((functionp dall-e-shell-openai-key)
+         (condition-case _err
+             (funcall dall-e-shell-openai-key)
+           (error
+            "KEY-NOT-FOUND")))
+        (t
+         nil)))
+
 (defun dall-e-shell--make-curl-request-command-list (request-data)
   "Build DALL-E curl command list using REQUEST-DATA."
   (append (list "curl" dall-e-shell--url)
@@ -442,12 +461,12 @@ ERROR-CALLBACK otherwise."
            (png-output (dall-e-shell-post-prompt
                         (concat "Please generate image for the following text: " image-description))))
       (goto-char (region-end))
-      (cond ((eq major-mode 'org-mode)
-             (insert (concat "\n\n[[file:" png-output "]]\n")))
-            ((eq major-mode 'markdown-mode)
-             (insert (concat "\n\n![](" png-output ")\n" )))
+      (cond ((derived-mode-p 'org-mode)
+             (insert "\n\n[[file:" png-output "]]\n"))
+            ((derived-mode-p 'markdown-mode)
+             (insert "\n\n![](" png-output ")\n"))
             (t (progn
-                 (insert (concat png-output "\n"))
+                 (insert png-output "\n")
                  (insert-image
                   (create-image png-output
                                 'png nil :width 400 :height 400))))))))
